var gCanvas = document.getElementById('id-canvas');

var CubeDisplay = (function (_canvas) {
	
	
	/*
	 * =============================
	 *            Canvas
	 * =============================
	 */

	 
	var _ctx = _canvas.getContext('2d');
	
	function drawCircle(ctx, coordinate, color, focus) {
		var h = coordinate[0] + X_POLE;
		var k = coordinate[1] + Y_POLE;
		
		ctx.beginPath();
		ctx.arc(h, k , 7, 0, 2 * Math.PI);
		ctx.fillStyle = color;
		ctx.fill();
		
		if(focus) {
			ctx.lineWidth = 2;
			ctx.strokeStyle = '#fff';
		} else {
			ctx.lineWidth = 1;
			ctx.strokeStyle = '#000';
		}
		ctx.stroke();
	}
	
	function clearCanvas(ctx) {
		ctx.clearRect(0, 0, _canvas.width, _canvas.height);
	}
	
	
	/*
	 * =============================
	 *    Projection and Rotation
	 * =============================
	 */
	
	
	var LEFT     = _canvas.offsetLeft;
	console.log(_canvas.offsetLeft);
	var TOP      = _canvas.offsetTop;
	console.log(_canvas.offsetTop);
	var X_POLE   = _canvas.width / 2;
	console.log(_canvas.width);
	var Y_POLE   = _canvas.height / 2;
	console.log(_canvas.height);
	var R_FACTOR = 60;
	
	function projection(camera, spawn) {
		var cameraX = camera.position[0];
		var cameraY = camera.position[1];
		var cameraZ = camera.position[2];
		
		var spawnX = spawn.position[0];
		var spawnY = spawn.position[1];
		var spawnZ = spawn.position[2];
		
		var x = ((spawnX - cameraX) * cameraZ / (cameraZ + spawnZ)) + cameraX;
		var y = ((spawnY - cameraY) * cameraZ / (cameraZ + spawnZ)) + cameraY;
		
		spawn.coordinate[0] = x;
		spawn.coordinate[1] = y;
	}
	
	function rotationXAxis(spawn, theta) {
		var z = spawn.position[2];
		var y = spawn.position[1];
		
		spawn.position[2] = z * Math.cos(theta * Math.PI / 180) - y * Math.sin(theta * Math.PI / 180);
		spawn.position[1] = z * Math.sin(theta * Math.PI / 180) + y * Math.cos(theta * Math.PI / 180);
	}
	
	function rotationZAxis(spawn, theta) {
		var x = spawn.position[0];
		var y = spawn.position[1];
		
		spawn.position[0] = x * Math.cos(theta * Math.PI / 180) + y * Math.sin(theta * Math.PI / 180);
		spawn.position[1] = y * Math.cos(theta * Math.PI / 180) - x * Math.sin(theta * Math.PI / 180);
	}
	
	function rotationYAxis(spawn, theta) {
		var x = spawn.position[0];
		var z = spawn.position[2];
		
		spawn.position[0] = x * Math.cos(theta * Math.PI / 180) - z * Math.sin(theta * Math.PI / 180);
		spawn.position[2] = x * Math.sin(theta * Math.PI / 180) + z * Math.cos(theta * Math.PI / 180);
	}
	
	function sortZAxis(spawn, spawnZSorted) {
		for(var i = 0, length = spawn.length; i < length; i++) {
			spawnZSorted[i] = [i, spawn[i].position[2]];
		}
		
		spawnZSorted.sort(function(left, right) {
			return left[1] - right[1];
		});
	}
	
	function deltaTheta(delta) {
		var theta = -1 * delta;
		
		if(Math.abs(theta) > 3) {
			theta = 3 * (theta / Math.abs(theta));
		}
		return theta;
	}
	
	function detectFocus(spawn, x, y) {
		var spawnX = spawn.coordinate[0] + X_POLE;
		var spawnY = spawn.coordinate[1] + Y_POLE;
		
		if(x < spawnX + 5 && x > spawnX - 5 && y < spawnY + 5 && y > spawnY - 5) {
			spawn.focus = !spawn.focus;
		}
		
		return spawn.focus;
	}
	
	
	/*
	 * =============================
	 *         Initilize
	 * =============================
	 */
	
	
	var _camera = new Camera();
	var _spawn = new Array();
	
	function Camera() {
		this.position = [0, 0, 0];
	}
	
	function Spawn(x, y, z, color, focus) {
		this.position = [x, y, z];
		this.coordinate = [0, 0];
		this.color = color;
		this.focus = focus;
	}
	
	function initialize() {
		var vertexPattern = new Array();
		var colorPattern = new Array();
		
		_camera.position[2] = -10 * R_FACTOR;
		
		vertexPattern = [-1.5 * R_FACTOR , -0.5 * R_FACTOR, 0.5 * R_FACTOR, 1.5 * R_FACTOR];
		colorPattern = [
			'#00ff00', '#08ff7c', '#58c2ff', '#0675ff',
			'#c7ff06', '#7aff2a', '#63a2c6', '#9468ce',
			'#ffef00', '#ffb820', '#944c7a', '#e052ff',
			'#ff2a00', '#ff2e5a', '#ff45d1', '#ff60ba'
		];
		
		var iSpawn = 0;
		for(var depth = 0; depth < 4; depth++) {
			var iColor = 0;
			for(var row = 0; row < 4; row++) {
				for(var col = 0; col < 4; col++) {
					_spawn[iSpawn] = new Spawn(
						vertexPattern[col],
						vertexPattern[depth],
						vertexPattern[row],
						colorPattern[iColor],
						false
					);
					
					iSpawn++;
					iColor++;
				}
			}
		}
		
		updateDisplay(_ctx, _camera, _spawn, 0, 0);
	}
	
	
	function initialPosition(spawn) {
		var vertexPattern = new Array();
		
		vertexPattern = [-1.5 * R_FACTOR, -0.5 * R_FACTOR, 0.5 * R_FACTOR, 1.5 * R_FACTOR];
		
		var iSpawn = 0;
		for(var depth = 0; depth < 4; depth++) {
			var iColor = 0;
			for(var row = 0; row < 4; row++) {
				for(var col = 0; col < 4; col++) {
					spawn[iSpawn].position[0] = vertexPattern[col];
					spawn[iSpawn].position[1] = vertexPattern[depth];
					spawn[iSpawn].position[2] = vertexPattern[row];
					
					iSpawn++;
				}
			}
		}
		
		updateDisplay(_ctx, _camera, _spawn, 0, 0);
	}
	
	function initialFocus(focus) {		
		for(var i = 0, length = _spawn.length; i < length; i++) {
			_spawn[i].focus = focus[i];
		}
		
		updateDisplay(_ctx, _camera, _spawn, 0, 0);
	}
	
	
	/*
	 * =============================
	 *         Update Event
	 * =============================
	 */
	
	
	function updateDisplay(ctx, camera, spawn, deltaX, deltaY) {
		clearCanvas(ctx);
		
		for(var i = 0, length = spawn.length; i < length; i++) {
			rotationXAxis(spawn[i], deltaTheta(-1 * deltaY));
			rotationYAxis(spawn[i], deltaTheta(deltaX));
			
			projection(camera, spawn[i]);
		}
		
		var spawnZSorted = new Array();
		
		sortZAxis(spawn, spawnZSorted);
		
		for(var i = 0, length = spawnZSorted.length; i < length; i++) {
			drawCircle(
				ctx,
				spawn[spawnZSorted[i][0]].coordinate,
				spawn[spawnZSorted[i][0]].color,
				spawn[spawnZSorted[i][0]].focus
			);
		}
	}
	
	function updateDetectFocus(ctx, spawn, x, y) {
		for(var i = 0, length = spawn.length; i < length; i++) {
			var focus = spawn[i].focus;
			if(focus != detectFocus(spawn[i], x - LEFT, y - TOP)) {
				var spawnZSorted = new Array();
		
				sortZAxis(spawn, spawnZSorted);
		
				for(var i = 0, length = spawnZSorted.length; i < length; i++) {
					drawCircle(
						ctx,
						spawn[spawnZSorted[i][0]].coordinate,
						spawn[spawnZSorted[i][0]].color,
						spawn[spawnZSorted[i][0]].focus
					);
				}
			}
		}
	}
	
	function updateMaskFocus(ctx, spawn) {
		clearCanvas(ctx);
		
		var spawnZSorted = new Array();
		
		sortZAxis(spawn, spawnZSorted);
		
		for(var i = 0, length = spawnZSorted.length; i < length; i++) {
			if(spawn[spawnZSorted[i][0]].focus) {
				drawCircle(
					ctx,
					spawn[spawnZSorted[i][0]].coordinate,
					spawn[spawnZSorted[i][0]].color,
					spawn[spawnZSorted[i][0]].focus
				);
			}
		}
	}
	
    /*
	 * =============================
	 *          Mouse Event
	 * =============================
	 */
	
	
	var _down = false;
	var _xDown = 0;
	var _yDown = 0;
	var _dblclickToggle = false;

	function mouseDown(e) {
		_down = true;
		_xDown = parseInt(e.clientX);
		_yDown = parseInt(e.clientY);

		updateDetectFocus(_ctx, _spawn, _xDown, _yDown);

		timeline_data[on_time-1].data = CubeDisplay.getFocus();
		$("#array_data").html(timeline_data[on_time-1].data);

		change_data_file("");

	}

	function mouseMove(e) {
		var x = parseInt(e.clientX);
		var y = parseInt(e.clientY);
		var deltaX = x - _xDown;
		var deltaY = y - _yDown;
		
		if(_down) {
			updateDisplay(_ctx, _camera, _spawn, deltaX, deltaY);
		}
		
		_xDown = x;
		_yDown = y;

		timeline_data[on_time-1].data = CubeDisplay.getFocus();
		$("#array_data").html(timeline_data[on_time-1].data);

		change_data_file("");

	}

	function mouseUp() {
		_down = false;

		timeline_data[on_time-1].data = CubeDisplay.getFocus();
		$("#array_data").html(timeline_data[on_time-1].data);

		change_data_file("");
	}

	function doubleClick() {
		if(_dblclickToggle) {
			initialPosition(_spawn);
		} else {
			updateMaskFocus(_ctx, _spawn);
		}
		
		_dblclickToggle = !_dblclickToggle;
	}
	
	function reSize() {
		LEFT = _canvas.offsetLeft;
		TOP = _canvas.offsetTop;
	}
	
	/*
	 * =============================
	 *        Event Listener
	 * =============================
	 */


	_canvas.addEventListener('dblclick', doubleClick, false);
	_canvas.addEventListener('mousedown', function(e) { mouseDown(e); }, false);
	_canvas.addEventListener('mousemove', function(e) { mouseMove(e); }, false);
	window.addEventListener('mouseup', mouseUp, false);
	window.addEventListener('resize', reSize, false);
	
	return {
		initial: function() {
			initialize();
		},
		
		getFocus: function() {
			var focus = new Array();
			
			for(var i = 0, length = _spawn.length; i < length; i++) {
				focus[i] = _spawn[i].focus ? 1 : 0;
			}
			return focus;
		},
		
		setFocus: function(focus) {
			initialFocus(focus);
		}
	};
	
}) (gCanvas);



function addActive(evt){
	on_time = evt;
	var index = "#time" + evt.toString();
	$(".timeline").removeClass("active");
	$(index).addClass("active");
	$("#array_data").html(timeline_data[evt-1].data);
	CubeDisplay.setFocus(timeline_data[evt-1].data);
}

function sleep(delay) {
        var start = new Date().getTime();
        while (new Date().getTime() < start + delay);
      }

function change_data_file(data) {
	
	var data_to_server = "";

	if(data == "")
	{
		data_to_server = "";
		for(var i = 0 ; i < max_number ; i++)
			{
			  	$("#array_data").html(timeline_data[i].data);
			  	 	//alert($("#array_data").html());
			  	data_to_server += $("#array_data").html();
			  	data_to_server += "\n";
			}
	}
	else
	{
	    	data_to_server = data;
	}
		  //alert(data_to_server);
	if(check_update)
	{
		  $.post( "writefile.php", { data : data_to_server });
	}

	$("#gettimedata").attr("rows",max_number);
	$("#gettimedata").val(data_to_server);

}

